/* Name : board.c
 *
 * Notes: This file contains all of the functions for creating, destroying,
 *        modifying, and querying Quixo(tm) game boards.
 *
 * $Log: board.cc $
 * Revision 1.3  1996/10/10 20:33:52  idr
 * Modified the class so that the ShiftRow() and ShiftCol() functions are
 * not a part of even the private interface.
 *
 * Revision 1.2  1996/10/08 19:47:12  idr
 * Added the FixMove() function.
 *
 * Revision 1.1  1996/10/01 18:38:58  idr
 * Initial revision
 */

#include <stdlib.h>
#include "board.h"

void ShiftRow( char board[5][5], int row, int col, int end, int player );
void ShiftCol( char board[5][5], int row, int col, int end, int player );

/*****************************************************************************/
/* Name : IsValidMove()
 *
 * Notes: This function is used to check that a set of source and destination
 *        coordinates define a valid move for the specified playing in the
 *        current game board configuration.
 */

BOOL Board::IsValidMove( move_t move, int player )
{
    size_t  source;


    /* In order for a move to be valid, the selected source piece must be
     * along the perimeter of the playing board.  This means that it must
     * come from a row or 0 or 4, or from a column of 0 or 4.
     */

    if ( (move.srcX == 0) || (move.srcX == 4) 
         || (move.srcY == 0) || (move.srcY == 4) )
    {
        source = board[ move.srcX ][ move.srcY ];

        /* The next test it to make sure that the selected piece is either
         * vacant or is owned by the current player.
         */

        if ( (source == EMPTY) || (source == player) )
        {
            /* The removed piece must be placed in either the same row or
             * column from which it was removed, and it must be placed
             * along the perimeter of the game board.
             */

            if ( ((move.destX == 0) || (move.destX == 4) 
	           || (move.destY == 0) || (move.destY == 4))
                 && ((move.srcX == move.destX) || (move.srcY == move.destY)) )
            {
                /* The final test is to make sure that the source and
                 * destination positions are not the same.
                 */

                return( !( (move.srcX == move.destX) 
                           && (move.srcY == move.destY) ) );
            }
        }
    }

    return( FALSE );
}




/*****************************************************************************/
/* Name :
 *
 * Notes:
 *
 */

BOOL Board::DoMove( move_t move, int player )
{
    /* First, make sure that the caller is requesting that a valid move
     * be made.
     */

    if ( ! IsValidMove( move, player ) )
        return( FALSE );

    if ( move.srcY == move.destY )
    {
        ShiftRow( board, move.srcY, move.srcX, move.destX, player );
    }
    else
    {
        ShiftCol( board, move.srcY, move.srcX, move.destY, player );
    }

    return( TRUE );
}




/*****************************************************************************/
/* Name : ShiftRow()
 *
 * Notes:
 *
 *        If the specified row contains "X O _ _ X", and col = 1 and end = 4,
 *        then the final result will be "X _ _ X O".
 */

void ShiftRow( char board[5][5], int row, int col, int end, int player )
{
    if ( end == 0 )
    {
        for ( /* empty */ ; col > 0 ; col-- )
        {
            board[ col ][ row ] = board[ col - 1 ][ row ] ;
        }
    }
    else
    {
        for ( /* empty */ ; col < 4 ; col++ )
        {
            board[ col ][ row ] = board[ col + 1 ][ row ];
        }
    }

    board[ end ][ row ] = player;
}




/*****************************************************************************/
/* Name : ShiftCol()
 *
 * Notes:
 *
 */

void ShiftCol( char board[5][5], int row, int col, int end, int player )
{
    if ( end == 0 )
    {
        for ( /* empty */ ; row > 0 ; row-- )
        {
            board[ col ][ row ] = board[ col ][ row - 1 ] ;
        }
    }
    else
    {
        for ( /* empty */ ; row < 4 ; row++ )
        {
            board[ col ][ row ] = board[ col ][ row + 1 ];
        }
    }

    board[ col ][ end ] = player;
}




/*****************************************************************************/
/* Name : IsWinner()
 *
 * Notes: This function is used to determin who the winner on a given game
 *        board is.  If there is no winner, it will return EMPTY.
 */

int Board::IsWinner( int player )
{
    int     sum;
    int     x;
    int     y;
    BOOL    player1Won;
    BOOL    player2Won;


    player1Won = FALSE;
    player2Won = FALSE;


    for ( x = 0 ; x < 5 ; x++ )
    {
        sum = 0;

        if ( player != EMPTY )
        {
            for ( y = 0 ; y < 5 ; y++ )
            {
                sum += board[ x ][ y ];
            }
        }

	player1Won |= (sum == -5);
	player2Won |= (sum ==  5);
    }




    for ( y = 0 ; y < 5 ; y++ )
    {
        sum = 0;

        if ( player != EMPTY )
        {
            for ( x = 0 ; x < 5 ; x++ )
            {
                sum += board[ x ][ y ];
            }
        }

	player1Won |= (sum == -5);
	player2Won |= (sum ==  5);
    }



    /* Finally, check the two diagonals.  This is done in much the same way
     * as the other two tests.
     */

    sum = 0;
    for ( x = 0 ; x < 5 ; x++ )
    {
        sum += board[ x ][ x ];
    }

    player1Won |= (sum == -5);
    player2Won |= (sum ==  5);




    sum = 0;
    for ( x = 0 ; x < 5 ; x++ )
    {
        sum += board[ x ][ 4 - x ];
    }

    player1Won |= (sum == -5);
    player2Won |= (sum ==  5);



    if ( player == PLAYER_1 )
    {
        if ( player2Won )
            return ( PLAYER_2 );

        if ( player1Won )
            return ( PLAYER_1 );
    }
    else
    {
        if ( player1Won )
            return ( PLAYER_1 );

        if ( player2Won )
            return ( PLAYER_2 );
    }

    return( EMPTY );
}




/*****************************************************************************/
/* Name : FixMove()
 *
 * Notes: This function is used to convert a move from the "x y side"
 *        representation to the format used in the move_t type.
 */

void FixMove( move_t * m, int x, int y, char side )
{
    m->srcX = x;
    m->srcY = y;

    switch( side )
    {
        case 't':
            m->destX = x;
            m->destY = 0;
            break;

        case 'b':
            m->destX = x;
            m->destY = 4;
            break;

        case 'l':
            m->destX = 0;
            m->destY = y;
            break;

        case 'r':
            m->destX = 4;
            m->destY = y;
            break;
    }
}

