/* Name : quixo.cc
 *
 * Notes:
 *
 * $Log: quixo.cc $
 * Revision 1.2  1996/10/08 19:47:12  idr
 * Move the FixMove() function to board.cc.  I also added the ability for
 * one of the player programs to be specified as "-".  If this is specified,
 * then the player is not a program, but a human.
 *
 * Revision 1.1  1996/10/07 22:24:59  idr
 * Initial revision
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <stdtypes.h>
#include "board.h"
#include "quixo.h"

void ShowUsage( const char * name );
void StartProgram( const char * program, FILE * fd[], const char * player );
void FixMove( move_t * m, int x, int y, int offset, char direction );
void InitPlayer( struct Player * p, const char * name, int playerNumber );

/*****************************************************************************/
/* Name : main()
 *
 * Notes:
 *
 */

int main( int argc, char ** argv )
{
    struct Player   player1;        /* Data for talking to player 1.        */
    struct Player   player2;        /* Data for talking to player 2.        */
    int             winner;         /* Who won the game?                    */
    Board           b;              /* Current game board.                  */
    move_t          m;              /* Last move that was made.             */
    int             totalMoves;     /* Total moves made in the game.        */



    if ( argc < 3 )
    {
        ShowUsage( argv[ 0 ] );
        exit( 1 );
    }


    /* Initialize the display in the input stuff. */

    OpenDisplay();
    InitInput();


    /* Create the structures for talking to our two players. */

    InitPlayer( & player1, argv[ 1 ], PLAYER_1 );
    InitPlayer( & player2, argv[ 2 ], PLAYER_2 );


    totalMoves = 0;
    ShowBoard( b );

    /* Play the game until somebody wins, or until they hit ^C. :) */

    while ( 1 )
    {
        /* Get the first player's move and do it.  If the move is not valid,
         * then display the move to the "console" and break out with player 2
         * as the winner.
         */

        m = player1.getMove( b, & player1 );

        totalMoves++;
        if ( ! b.DoMove( m, PLAYER_1 ) )
        {
   fprintf(stdout,"Player 1 made an invalid move and loses!\n");
   DisplayMove(&m, NULL); 
            winner = PLAYER_2;
            break;
	}

        DisplayMove( & m, NULL );
        ShowBoard( b );

        /* It that move caused somebody to win the game, then we'll need to
         * break out of the loop.  The game is over.
         */

        winner = b.IsWinner( PLAYER_1 );
        if ( winner != EMPTY ) break;


        /* Tell player 2 *exacly* what player 1 did. */

        player2.showMove( & m, & player2 );



        /* Get the second player's move and do it.  If the move is not valid,
         * then display the move to the "console" and break out with player 1
         * as the winner.
         */

        m = player2.getMove( b, & player2 );

        totalMoves++;
        if ( ! b.DoMove( m, PLAYER_2 ) )
        {
   fprintf(stdout,"Player 2 made an invalid move and loses!\n");
   DisplayMove(&m, NULL); 
            winner = PLAYER_1;
            break;
	}

        DisplayMove( & m, NULL );
        ShowBoard( b );

        /* It that move caused somebody to win the game, then we'll need to
         * break out of the loop.  The game is over.
         */

        winner = b.IsWinner( PLAYER_2 );
        if ( winner != EMPTY ) break;


        /* Tell player 1 *exacly* what player 2 did. */

        player1.showMove( & m, & player1 );
    }


    /* Tell them who won, close up shop, and terminate. */

    DisplayWinner( winner, totalMoves );
    CloseDisplay();
    return( 0 );
}




/*****************************************************************************/
/* Name : InitPlayer()
 *
 * Notes: This function initializes the player structures.  The the name of
 *        the "file" assosicated with the player is "-", then the player is
 *        actually a human player.  If not, then we'll need to start the
 *        specified program to play as this player.
 */

void InitPlayer( struct Player * p, const char * name, int playerNumber )
{
    p->player = playerNumber;

    if ( strcmp( "-", name ) == 0 )
    {
        p->getMove = GetHumanMove;
        p->showMove = DisplayMove;
    }
    else
    {
        p->getMove = GetComputerMove;
        p->showMove = PutComputerMove;
        StartProgram( name, p->fd, (playerNumber == -1) ? "1" : "2" );
    }
}




/*****************************************************************************/
/* Name : StartProgram()
 *
 * Notes: This function starts a new process and runs the named program.  It
 *        fills fd[0] with a file handle that is tied to the new program's
 *        STDOUT (i.e., you can read the output of the child program through
 *        this file), and fills fd[1] with a file handle that is tied to the
 *        new program's STDIN.
 */

void StartProgram( const char * program, FILE * fd[], const char * player )
{
    int     pipeIn[ 2 ];        /* Provides STDIN for the CHILD.    */
    int     pipeOut[ 2 ];       /* Provides STDOUT for the CHILD.   */


    /* First, create the two pipes that we will use to communicate with the
     * child processes.
     */

    if ( pipe( pipeIn ) == -1 )
    {
        perror( "pipe( pipeIn )" );
        exit( 1 );
    }

    if ( pipe( pipeOut ) == -1 )
    {
        perror( "pipe( pipeOut )" );
        exit( 1 );
    }



    /* Create the child process! */

    if ( fork() == 0 )
    {
        /* Close the default STDIN and STDOUT file handles.  We want our
         * STDIN and STDOUT tied directly to our parent process.
         */

        close( STDIN_FILENO );
        close( STDOUT_FILENO );


        /* Close the write part of our STDIN pipe and the read part of our
         * STDOUT pipe.  As the child, we can't use these.
         */

        close( pipeIn[ 1 ] );
        close( pipeOut[ 0 ] );



        if ( dup2( pipeIn[ 0 ], STDIN_FILENO ) == -1 )
        {
            perror( "dup2( pipeIn[ 0 ], STDIN_FILENO )" );
            exit( 1 );
        }

        if ( dup2( pipeOut[ 1 ], STDOUT_FILENO ) == -1 )
        {
            perror( "dup2( pipeOut[ 1 ], STDOUT_FILENO )" );
            exit( 1 );
        }


        /* Run the specified program. */

        execl( program, program, player, NULL );


        /* If we get to this point, it means that execl() failed.  Let's
         * print an error message and exit.
         */

        perror( "execl()" );
        exit( 1 );
    }




    /* Close the read part of the child's STDIN pipe and the out part of
     * the child's STDOUT pipe.  As the parent, we (can't) won't be using
     * these parts.
     */

    close( pipeIn[ 0 ] );
    close( pipeOut[ 1 ] );


    /* Convert the parts of the pipes that we will be using to normal
     * stdio style FILEs.
     */

    fd[ 0 ] = fdopen( pipeOut[ 0 ], "r" );
    if ( fd[ 0 ] == NULL )
    {
        perror( "fdopen" );
        exit( 1 );
    }

    fd[ 1 ] = fdopen( pipeIn[ 1 ], "w" );
    if ( fd[ 1 ] == NULL )
    {
        perror( "fdopen" );
        exit( 1 );
    }
}




/*****************************************************************************/
/* Name : ShowUsage()
 *
 * Notes: This function attempts to show the user how to properly run our
 *        little program.
 */

void ShowUsage( const char * name )
{
    fprintf( stderr, "Usage: %s player1_program player2_program\n" \
                     "    If the player_program is \"-\", then the player " \
                     "actually a human player.\n",
             name );
}
